from enum import Enum


class Region(Enum):
    # Countries
    RUSSIA = "225"
    UKRAINE = "187"
    BELARUS = "149"
    KAZAKHSTAN = "159"

    # Russian regions
    ARKHANGELSK = "20"
    ASTRAKHAN = "37"
    BARNAUL = "197"
    BELGOROD = "4"
    BLAGOVESHCHENSK = "77"
    BRYANSK = "191"
    VELIKY_NOVGOROD = "24"
    VLADIVOSTOK = "75"
    VLADIKAVKAZ = "33"
    VLADIMIR = "192"
    VOLGOGRAD = "38"
    VOLOGDA = "21"
    VORONEZH = "193"
    GROZNY = "1106"
    EKATERINBURG = "54"
    IVANOVO = "5"
    IRKUTSK = "63"
    YOSHKAR_OLA = "41"
    KAZAN = "43"
    KALININGRAD = "22"
    KEMEROVO = "64"
    KOSTROMA = "7"
    KRASNODAR = "35"
    KRASNOYARSK = "62"
    KURGAN = "53"
    KURSK = "8"
    LIPETSK = "9"
    MAKHACHKALA = "28"
    MOSCOW_AND_MOSCOW_REGION = "1"
    MOSCOW = "213"  # Note: Documentation shows both 1 and 213 for Moscow
    MURMANSK = "23"
    NALCHIK = "30"
    NAZRAN = "1092"
    NIZHNY_NOVGOROD = "47"
    NOVOSIBIRSK = "65"
    OMSK = "66"
    ORYOL = "10"
    ORENBURG = "48"
    PENZA = "49"
    PERM = "50"
    PSKOV = "25"
    ROSTOV_ON_DON = "39"
    RYAZAN = "11"
    SAMARA = "51"
    SAINT_PETERSBURG = "2"
    SARANSK = "42"
    SMOLENSK = "12"
    SOCHI = "239"
    STAVROPOL = "36"
    SURGUT = "973"
    TAMBOV = "13"
    TVER = "14"
    TOMSK = "67"
    TULA = "15"
    ULYANOVSK = "195"
    UFA = "172"
    KHABAROVSK = "76"
    CHEBOKSARY = "45"
    CHELYABINSK = "56"
    CHERKESSK = "1104"
    YAROSLAVL = "16"